package com.ekexiu.portal.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.Dynamic;
import org.jfw.apt.orm.annotation.dao.method.From;
import org.jfw.apt.orm.annotation.dao.method.OrderBy;
import org.jfw.apt.orm.annotation.dao.method.SetSentence;
import org.jfw.apt.orm.annotation.dao.method.operator.DeleteWith;
import org.jfw.apt.orm.annotation.dao.method.operator.Insert;
import org.jfw.apt.orm.annotation.dao.method.operator.LimitSelect;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectList;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectOne;
import org.jfw.apt.orm.annotation.dao.method.operator.Update;
import org.jfw.apt.orm.annotation.dao.method.operator.UpdateWith;
import org.jfw.apt.orm.annotation.dao.param.LessThan;
import org.jfw.apt.orm.annotation.dao.param.Set;
import org.jfw.util.PageQueryResult;

import com.ekexiu.portal.po.Article;
import com.ekexiu.portal.pojo.FindInfo;

@DAO
public abstract class ArticleDao {
	@Insert
	public abstract int insert(Connection con, Article article) throws SQLException;
	
	@Update
	@Dynamic
	public abstract int update(Connection con, Article article) throws SQLException;
	
	@UpdateWith
	@From(Article.class)
	@SetSentence("ARTICLE_AGREE = ARTICLE_AGREE + 1")
	public abstract int incAgree(Connection con,String articleId) throws SQLException;
	
	@UpdateWith
	@From(Article.class)
	@SetSentence("ARTICLE_AGREE = ARTICLE_AGREE - 1")
	public abstract int decAgree(Connection con,String articleId) throws SQLException;
	
	@UpdateWith
	@From(Article.class)
	public abstract int updateSubject(Connection con, @Set String subject, String articleId) throws SQLException;
	
	@UpdateWith
	@From(Article.class)
	public abstract int updateIndustry(Connection con, @Set String industry, String articleId) throws SQLException;
	
	@DeleteWith
	@From(Article.class)
	public abstract int delete(Connection con, String articleId) throws SQLException;
	
	@DeleteWith
	@From(Article.class)
	public abstract int deletePro(Connection con, String professorId) throws SQLException;
	
	@SelectList
	@OrderBy(" ORDER BY MODIFY_TIME DESC ")
	public abstract List<Article> queryPro(Connection con, String professorId) throws SQLException;
	
	@SelectList
	@OrderBy(" ORDER BY MODIFY_TIME DESC ")
	public abstract List<Article> queryOrg(Connection con, String orgId) throws SQLException;
	
	@LimitSelect
	@OrderBy(" ORDER BY MODIFY_TIME DESC ")
	public abstract List<Article> queryLimit(Connection con,String orgId,@LessThan String modifyTime,int rows)throws SQLException;
	
	@SelectOne
	@Nullable
	public abstract Article queryOne(Connection con, String articleId) throws SQLException;
	
	public PageQueryResult<FindInfo> queryPage(Connection con,int pageSize,int pageNo) throws SQLException{
        int total = 0;
        PageQueryResult<FindInfo> queryResult = new PageQueryResult<FindInfo>();
        String sql = null;
        sql = "SELECT SUM(NUM) FROM (SELECT COUNT(1) NUM FROM ARTICLE UNION ALL SELECT COUNT(1) NUM FROM RESOURCE) TBL";
        PreparedStatement ps = con.prepareStatement(sql);
        try{
        	queryResult.setPageSize(pageSize);
            ResultSet rs = ps.executeQuery();
            try{
                rs.next();
                total = rs.getInt(1);
            }finally{
                try{rs.close();}catch(Exception e1){}
            }
        }finally{
            try{ps.close();}catch(Exception e2){}
        }
        queryResult.setTotal(total);
        if(0== total){
        	queryResult.setPageNo(1);
        	queryResult.setData(Collections.<FindInfo>emptyList());
            return queryResult;
        }
        boolean firstPage = (1 == pageNo);
        if(firstPage){
        	queryResult.setPageNo(1);
        	sql = "SELECT ARTICLE_ID,ARTICLE_TITLE,ARTICLE_IMG,ARTICLE_TYPE,CREATE_TIME,ORG_ID,PROFESSOR_ID FROM ARTICLE"
                	+ " UNION ALL SELECT RESOURCE_ID,RESOURCE_NAME,NULL,'3',CREATE_TIME,NULL,PROFESSOR_ID FROM RESOURCE";
            sql = sql + " ORDER BY CREATE_TIME DESC ";
            sql = sql + " LIMIT " + pageSize;
        }else{
            int pageNum = total / pageSize;
            if(total % pageSize != 0){
                ++pageNum;
            }
            if(pageNo > pageNum){
                pageNo = pageNum;
            }
            queryResult.setPageNo(pageNo);
            --pageNo;
            int offset = (pageNo * pageSize);
            sql = "SELECT ARTICLE_ID,ARTICLE_TITLE,ARTICLE_IMG,ARTICLE_TYPE,CREATE_TIME,ORG_ID,PROFESSOR_ID FROM ARTICLE"
                	+ " UNION ALL SELECT RESOURCE_ID,RESOURCE_NAME,NULL,'3',CREATE_TIME,NULL,PROFESSOR_ID FROM RESOURCE";
            sql = sql + " ORDER BY CREATE_TIME DESC ";
            sql = sql + " LIMIT " + pageSize+ " OFFSET " + offset;
        }
        ps = con.prepareStatement(sql);
        try{
            ResultSet rs = ps.executeQuery();
            try{
                List<FindInfo> findInfos = new ArrayList<FindInfo>();
                queryResult.setData(findInfos);
                int size = 0;
                while((size<pageSize) && rs.next()){
                    ++size;
                    FindInfo findInfo =  new FindInfo();
                    findInfo.setId(rs.getString(1));
                    findInfo.setName(rs.getString(2));
                    String image = rs.getString(3);
                    if(rs.wasNull()){
                    	image = null;
                    }
                    findInfo.setImage(image);
                    findInfo.setType(rs.getString(4));
                    findInfo.setCreateTime(rs.getString(5));
                    String orgId = rs.getString(6);
                    if(rs.wasNull()){
                    	findInfo.setOwner(rs.getString(7));
                    }else{
                    	findInfo.setOwner(orgId);
                    }
                    findInfos.add(findInfo);
                }
                return queryResult;
            }finally{
                try{rs.close();}catch(Exception e3){}
            }
        }finally{
            try{ps.close();}catch(Exception e4){}
        }
    }

}
